#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <commctrl.h>

#include "ahx/AHX.h"
#include "ahx/Windows.h"
#include "Debug.h"
#include "Main.h"
#include "resource.h"

BOOL CALLBACK AHXAboutDialog::_DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
	static AHXAboutDialog* This = NULL;
	if(uMsg == WM_INITDIALOG) This = (AHXAboutDialog*)lParam;
	return This->DialogProc(hDlg, uMsg, wParam, lParam);
}

BOOL AHXAboutDialog::DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	OutputDebugString("DialogProc\n");
	switch(message) {
		case WM_INITDIALOG: {
			hDialog = hDlg;
			OutputDebugString("InitDialog()\n");
			InitScroller();

			TIMECAPS tc;
			CHECK(timeGetDevCaps(&tc, sizeof(TIMECAPS)) == TIMERR_NOERROR, "timeGetDevCaps()");
			iTimerRes = min(max(tc.wPeriodMin, 1), tc.wPeriodMax);
			CHECK(timeBeginPeriod(iTimerRes) == TIMERR_NOERROR, "timeBeginPeriod()");
			iTimerId = timeSetEvent(60, iTimerRes, _TimerCallback, (DWORD)this, TIME_PERIODIC);
			CHECK(iTimerId, "timeSetEvent()");
		}   break;
		case WM_DRAWITEM:
			switch((UINT)wParam) {
				case IDC_SCROLLER: RenderScroller(); break;
			}
			break;
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case IDOK: EndDialog(hDlg, TRUE); break;
			}
			break;
		case WM_DESTROY:
			if(iTimerId) { timeKillEvent(iTimerId); iTimerId = 0; }
			if(iTimerRes) { timeEndPeriod(iTimerRes); iTimerRes = 0; }
			break;
	}
	return FALSE;
}

void AHXAboutDialog::Show(HWND hParent)
{
	int i = DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ABOUT), hParent, (DLGPROC)_DialogProc, (LPARAM)this);
	OutputDebugString(i==-1?"Failed!\n":"Okay\n");
}

void AHXAboutDialog::InitScroller()
{
	char* Scroller[] = {"","","","","","","","","","","","","","",
		"/WinAHX 1.0",
		" 1998 Abyss",
		"","","",
		"/Copyright",
		"WinAHX and the AHX Replayer, although copyrighted",
		"by Abyss, are Freeware! You may spread them as you",
		"like - as long as you don't charge any money for it!",
		"If you want to put WinAHX on CD-ROM, the only thing",
		"we ask is that you send us a copy of that CD-ROM!",
		"","","",
		"Code",
		"/Bartman",
		"",
		"Quality Assurance",
		"/Pink",
		"",
		"Original Amiga Code",
		"/Dexter",
		"",
		"Original Concept",
		"/Pink",
		"",
		"Beta testing",
		"/Pink, Grace, Poet",
		"","","",
		"/For updates check",
		"http://abyss.moving-people.net/ahxpc.html",
		"",
		"/SDK available at",
		"http://abyss.moving-people.net/ahxpc.html",
		"",
		"",
		"",
		"/Features",
		"> 100% accurate playback of all AHX 1.x/2.x tunes",
		"> quality approved by Pink && Dexter, the AHX creators",
		"> 100% C++ code, thus easily portable",
		"> very low CPU usage (<4% on P200)",
		"> Player available for use in own productions",
		"> Small && intuitive user interface",
		"> Coded by Bartman/Abyss",
		"> It's free! It's small!",
		"",
		"",
		"",
		"/Ports",
		"We're searching for skilled coders that like to",
		"port the AHX Player (pure C++) to different OSs:",
		"BeOS, MacOS, Unix (Linux, HP-UX, SunOS), Java,",
		"or any other platform capable of playing PCM sound",
		"(PDAs not excluded)",
		"",
		"Note that we won't accept porting the player to Delphi",
		"",
		"We're also interested in coders writing plug-ins for",
		"various players esp. DirectShow and WinAmp",
		"",
		"If you're interested, please write to Bartman at",
		"/barto@gmx.net",
		"and include some references of your skill...",
		"",
		"",
		"",
		"/Source code",
		"The AHX-Player source is strictly confidential.",
		"Only coders of a port will get the source,",
		"if you just want to add capability of playing",
		"AHX tunes in your application, intro or demo",
		"download the SDK from our homepage, it includes",
		"the necessary docs, link-library (Visual C++)",
		"and sample sources",
		"",
		"",
		"",
		"/For more informations",
		"consult the readme/documentation and turn",
		"to our homepage",
		"http://abyss.moving-people.net/ahxpc.html",
		"","","","","","","","","","","","","","",
		"\\"
	};
	
	iScrollPos = 0;

	HFONT hFont = CreateFont( -8,0,0,0,FW_NORMAL,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_DONTCARE,"MS Sans Serif");
	HFONT hFont2 = CreateFont( -14,0,0,0,FW_BOLD,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH|FF_DONTCARE,"MS Sans Serif");

	RECT r;
	GetClientRect(GetDlgItem(hDialog, IDC_SCROLLER), &r);
	HDC dc = GetDC(hDialog); 
	hScroller = CreateCompatibleBitmap(dc, r.right, 1850); 
	ReleaseDC(hDialog, dc);

	HBRUSH winBrush = GetSysColorBrush(COLOR_BTNFACE);

	HDC memDC = CreateCompatibleDC(NULL);
	HBITMAP oldBM = (HBITMAP)SelectObject(memDC, hScroller);
	HFONT oldFont = (HFONT)SelectObject(memDC, hFont);

	r.bottom = 1850;

	FillRect(memDC, &r, winBrush);
	SetBkMode(memDC, TRANSPARENT);
	for(int i = 0; Scroller[i][0] != '\\'; i++) {
		if(Scroller[i][0] == '/') {
			SelectObject(memDC, hFont2);
			DrawText(memDC, &Scroller[i][1], -1, &r, DT_CENTER);
			r.top += 16;
		} else {
			SelectObject(memDC, hFont);
			DrawText(memDC, Scroller[i], -1, &r, DT_CENTER);
			r.top += 14;
		}
	}

	SelectObject(memDC, oldBM);
	SelectObject(memDC, oldFont);
	DeleteObject(hFont);
	DeleteDC(memDC);
}

void AHXAboutDialog::RenderScroller()
{
	HDC hDC = GetDC(GetDlgItem(hDialog, IDC_SCROLLER));
	RECT r; GetClientRect(GetDlgItem(hDialog, IDC_SCROLLER), &r);

	HDC memDC = CreateCompatibleDC(NULL);
	HBITMAP oldBM = (HBITMAP)SelectObject(memDC, hScroller);
	
	BitBlt(hDC, 0, 0, r.right, r.bottom, memDC, 0, iScrollPos, SRCCOPY);
	
	SelectObject(memDC, oldBM);
	DeleteDC(memDC);

	ReleaseDC(GetDlgItem(hDialog, IDC_SCROLLER), hDC);
}

void FAR PASCAL AHXAboutDialog::_TimerCallback(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	AHXAboutDialog* This = (AHXAboutDialog*)dwUser;
	This->RenderScroller();
	This->iScrollPos++;
	This->iScrollPos %= 1450;
}
