#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <commctrl.h>

#include "ahx/AHX.h"
#include "ahx/Windows.h"
#include "Debug.h"
#include "Main.h"
#include "resource.h"

AHXProperties Properties;
AHXDialog Dialog;
AHXInfoDialog InfoDialog;

static char CmdLine[256] = "";
HWND theWindow = 0;

static void SplitCommandLine()
{
	char* lpszCommandLine = GetCommandLine();
	int i=0;
    if(*lpszCommandLine == '"') {
        while ((*(++lpszCommandLine) != '"') && (*lpszCommandLine != '\0') ) {}
        if (*lpszCommandLine == '"') lpszCommandLine++;
    } else {
        while (*lpszCommandLine > ' ') lpszCommandLine++;
    }
    while (*lpszCommandLine && (*lpszCommandLine <= ' ' || *lpszCommandLine == '"')) lpszCommandLine++;
	strcpy(CmdLine, lpszCommandLine);
	if(CmdLine[strlen(CmdLine)-1] == '"') CmdLine[strlen(CmdLine)-1] = '\0';
}

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	char title[256];

	if(GetWindowText(hwnd, title, 256)) {
		if(strstr(title, "WinAHX 1.0")) theWindow = hwnd;
	}
	return (theWindow == 0);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	SplitCommandLine();

	// look if we find an existing WinAHX running
	EnumWindows((WNDENUMPROC)EnumWindowsProc, 0);
	if(strlen(CmdLine) && theWindow) {
		// we got one, so send the name of the song to the existing window!
		SendMessage(theWindow, WM_USER+10, 1, ~1);
		SendMessage(theWindow, WM_USER+10, strlen(CmdLine), ~strlen(CmdLine));
		for(unsigned int i = 0; i < strlen(CmdLine); i++)
			SendMessage(theWindow, WM_USER+10, CmdLine[i], ~CmdLine[i]);
		SendMessage(theWindow, WM_USER+10, 0, ~0);
		return 0;
	}
	
	Dialog.Init();

	return 0;
}

void AHXDialog::Init()
{
	Player = NULL;
	
	InitCommonControls();
	DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_MAIN), NULL, (DLGPROC)_AHXDlgProc, (LPARAM)this);
}

long AHXDialog::AHXDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static char RemoteFile[256];
	static char* RemoteFilePtr = RemoteFile;
	static int RemoteLength = 0;
	static int RemoteInit = 0;

	switch(message) {
		case WM_USER+10: {
			// discard non-encoded messages
			if((unsigned)lParam != ~(unsigned)wParam) break;
			if(wParam == 1 && RemoteInit == 0) { RemoteInit = 1; RemoteFilePtr = RemoteFile; break; }
			if(RemoteInit == 1) { RemoteInit = 2; RemoteLength = wParam; break; }
			if(RemoteInit == 2 && RemoteLength > 0) {
				*RemoteFilePtr++ = wParam;
				if(wParam == 0) {
					// now we should have a valid name
					LoadSong(RemoteFile);
					RemoteInit = 0;
				}
			}
		} break;
		case WM_MOVE: {
			if(!Properties.Dock) break;

			WINDOWPLACEMENT pl;

			pl.length = sizeof(WINDOWPLACEMENT); GetWindowPlacement(hDlg, &pl);

			SetWindowPos(InfoDialog.hInfoDlg, NULL, pl.rcNormalPosition.right, pl.rcNormalPosition.top, 0, 0, SWP_NOSIZE|SWP_NOZORDER|SWP_NOACTIVATE);
		} break;
		case WM_INITDIALOG: {
			hDialog = hDlg;
	
			InfoDialog.Init(hDlg);

			char Ini[256];
			int x, y;

			GetModuleFileName(NULL, Ini, 256); strcpy(&Ini[strlen(Ini)-3], "ini");

			x = GetPrivateProfileInt("Main Window", "left", 0, Ini);
			y = GetPrivateProfileInt("Main Window", "top", 0, Ini);
			SetWindowPos(hDlg, NULL, x, y, 0, 0, SWP_NOSIZE|SWP_NOZORDER|SWP_NOACTIVATE);
			
			hBtnBitmaps[0] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_BACKWD));
			hBtnBitmaps[1] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_EJECT));
			hBtnBitmaps[2] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_FWD));
			hBtnBitmaps[3] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_PAUSE));
			hBtnBitmaps[4] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_PLAY));
			hBtnBitmaps[5] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_SKIPBACK));
			hBtnBitmaps[6] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_SKIPFWD));
			hBtnBitmaps[7] = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_STOP));
			hBtnIcons[0]   = LoadIcon  (GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_OPEN));
			hBtnIcons[1]   = LoadIcon  (GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_PROPERTIES));
			hTimeFont      = LoadBitmap(GetModuleHandle(NULL), MAKEINTRESOURCE(IDB_TIMEFONT));
			HDC dc; dc = GetDC(hDlg); hTimeBitmap    = CreateCompatibleBitmap(dc, 77, 42); ReleaseDC(hDlg, dc);
			iTime = -1; iTimeMode = iTimerRes = iTimerId = 0; RenderTime(); SyncTitle(); SyncButtons(0);
			SendMessage(GetDlgItem(hDlg, IDC_VOLUME), TBM_SETRANGE, TRUE, MAKELONG(0, 64));
			SendMessage(GetDlgItem(hDlg, IDC_VOLUME), TBM_SETPOS, TRUE, 64);

			SendMessage(hDlg, WM_SETICON, TRUE, (LPARAM)LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1)));
			DeleteMenu(GetSystemMenu(hDlg, FALSE), SC_SIZE, MF_BYCOMMAND);
			DeleteMenu(GetSystemMenu(hDlg, FALSE), SC_MAXIMIZE, MF_BYCOMMAND);
			AppendMenu(GetSystemMenu(hDlg, FALSE), MF_SEPARATOR, 0, NULL);
			AppendMenu(GetSystemMenu(hDlg, FALSE), MF_STRING, IDM_OPTIONS, "Options...");
			AppendMenu(GetSystemMenu(hDlg, FALSE), MF_SEPARATOR, 0, NULL);
			AppendMenu(GetSystemMenu(hDlg, FALSE), MF_STRING, IDM_ABOUT, "About WinAHX...");

			int MixRates[4] = { 8000, 11025, 22050, 44100 };
			CHECK(WaveOut.Init(MixRates[Properties.MixRate], (Properties.Output+1)*8, 2, Properties.BufSize, ((float)Properties.Boost)*.1f), "Can't initialize Wave Output");
			WaveOut.SetOption(AHXOI_OVERSAMPLING, Properties.Oversampling);
			CHECK(WaveOut.StartBackgroundPlay(), "Couldn't start background play");

			if(strlen(CmdLine)) LoadSong(CmdLine);
							
		} break;
		case WM_DRAWITEM:
			int bm;
			switch((UINT)wParam) {
				case IDC_BACKWD:    bm =    0; break;
				case IDC_CLOSE:     bm =    1; break;
				case IDC_SKIPBACK:  bm =    5; break;
				case IDC_PLAYPAUSE: bm =    4; break;
				case IDC_SKIPFWD:   bm =    6; break;
				case IDC_STOP:      bm =    7; break;
				case IDC_FWD:       bm =    2; break;
				case IDC_TIME:      bm = 1000; break;
				case IDC_OPEN:      bm =   -1; break;
				case IDC_PROPERTIES:bm =   -2; break;
				default:            bm = 1;
			}
			DrawNoFocusButton((UINT)wParam, bm, (LPDRAWITEMSTRUCT)lParam);
			break;
		case WM_COMMAND:
			if(HIWORD(wParam) != BN_CLICKED) break;
			switch(LOWORD(wParam)) {
				case IDC_PROPERTIES: InfoDialog.Show(); break;
				case IDC_OPEN: LoadSong(); break;
				case IDC_STOP: WaveOut.Stop(); Player->InitSubsong(Subsong); break;
				case IDC_PLAYPAUSE: 
					if(WaveOut.Playing && !WaveOut.Paused) { WaveOut.Pause(); break; }
					if(WaveOut.Playing &&  WaveOut.Paused) { WaveOut.Resume(); break; }
					if(!WaveOut.Playing) WaveOut.Play(Player); 
					break;
				case IDC_CLOSE:
					CloseSong();
					break;
				case IDC_SKIPFWD:
					Subsong++;
					WaveOut.Stop();
					Player->InitSubsong(Subsong);
					WaveOut.Play(Player);
					SyncButtons(1);
					break;
				case IDC_SKIPBACK:
					Subsong--;
					WaveOut.Stop();
					Player->InitSubsong(Subsong);
					WaveOut.Play(Player);
					SyncButtons(1);
					break;
				case IDC_FWD:
					Player->NextPosition();
					break;
				case IDC_BACKWD:
					Player->PrevPosition();
					break;
				case IDC_TIME: /*iTimeMode++; iTimeMode %= 2;
					if(iTimeMode == 1) {
						TIMECAPS tc;
						CHECK(timeGetDevCaps(&tc, sizeof(TIMECAPS)) == TIMERR_NOERROR, "timeGetDevCaps()");
						iTimerRes = min(max(tc.wPeriodMin, 1), tc.wPeriodMax);
						CHECK(timeBeginPeriod(iTimerRes) == TIMERR_NOERROR, "timeBeginPeriod()");
						iTimerId = timeSetEvent(25, iTimerRes, _TimerCallback, (DWORD)this, TIME_PERIODIC);
						CHECK(iTimerId, "timeSetEvent()");
					} else {
						if(iTimerId) { timeKillEvent(iTimerId); iTimerId = 0; }
						if(iTimerRes) { timeEndPeriod(iTimerRes); iTimerRes = 0; }
						RenderTime(); DrawTime();
					}*/
					break;
			}
			break;
		case WM_SYSCOMMAND: {
			switch(wParam & 0xFFF0) {
				case IDM_OPTIONS: {
					Properties.Show(hDlg);
				}   break;
				case IDM_ABOUT: {
					AHXAboutDialog About;
					About.Show(hDlg);
				}   break;
			}
		}   break;
		case WM_HSCROLL: 
			if((HWND)lParam == GetDlgItem(hDlg, IDC_VOLUME)) {
				int dwPos = SendMessage(GetDlgItem(hDlg, IDC_VOLUME), TBM_GETPOS, 0, 0);
				WaveOut.SetVolume(dwPos);
			}
			break;
		case WM_TIMER:
			iTime = 0;
			if(iTimeMode == 0) { RenderTime(); DrawTime(); }
			SyncTitle(); 
			break;
		case WM_CLOSE: {
			char Ini[256], Str[64];
			WINDOWPLACEMENT pl;

			pl.length = sizeof(WINDOWPLACEMENT); GetWindowPlacement(hDlg, &pl);
			GetModuleFileName(NULL, Ini, 256); strcpy(&Ini[strlen(Ini)-3], "ini");

			_snprintf(Str, 64, "%d", pl.rcNormalPosition.left); WritePrivateProfileString("Main Window", "left", Str, Ini);
			_snprintf(Str, 64, "%d", pl.rcNormalPosition.top); WritePrivateProfileString("Main Window", "top", Str, Ini);

			pl.length = sizeof(WINDOWPLACEMENT); GetWindowPlacement(InfoDialog.hInfoDlg, &pl);

			_snprintf(Str, 64, "%d", InfoDialog.bVisible); WritePrivateProfileString("Info Window", "Visible", Str, Ini);
			_snprintf(Str, 64, "%d", pl.rcNormalPosition.left); WritePrivateProfileString("Info Window", "left", Str, Ini);
			_snprintf(Str, 64, "%d", pl.rcNormalPosition.top); WritePrivateProfileString("Info Window", "top", Str, Ini);
			_snprintf(Str, 64, "%d", pl.rcNormalPosition.right); WritePrivateProfileString("Info Window", "right", Str, Ini);
			_snprintf(Str, 64, "%d", pl.rcNormalPosition.bottom); WritePrivateProfileString("Info Window", "bottom", Str, Ini);

			EndDialog(hDlg, TRUE);
		} break;
		case WM_DESTROY:
			if(iTimerId) { timeKillEvent(iTimerId); iTimerId = 0; }
			if(iTimerRes) { timeEndPeriod(iTimerRes); iTimerRes = 0; }
			WaveOut.Stop();
			VERIFY(WaveOut.StopBackgroundPlay());
			break;
	}
	return FALSE;
}

void AHXDialog::SyncTitle()
{
	char Title[256];

	if(Player) {
		if(iTime != -1) sprintf(Title, "%s [%s] - WinAHX 1.0", Player->Song.Name, sTime);
		           else sprintf(Title, "%s - WinAHX 1.0", Player->Song.Name);
	} else              sprintf(Title, "WinAHX 1.0");
	SetWindowText(hDialog, Title);
}

void AHXDialog::LoadSong(char* Name)
{
	OPENFILENAME of;
	char Filename[256] = "";
	memset(&of, 0, sizeof(OPENFILENAME));
	of.lStructSize = sizeof(OPENFILENAME);
	of.lpstrFilter = "AHX Songs (*.AHX; AHX.*)\0*.AHX;AHX.*\0\0";
	of.lpstrFile = Filename;
	of.nMaxFile = 256;
	of.Flags = OFN_EXPLORER|OFN_FILEMUSTEXIST|OFN_LONGNAMES;

	if(!strlen(Name)) {
		if(!GetOpenFileName(&of)) return;
	}

	if(Player) {
		WaveOut.Stop();
		delete Player;
		Player = NULL;
	}
	
	Player = new AHXPlayer;
	Player->Init(&Waves);
	int ret;
	if(!strlen(Name)) ret = Player->LoadSong(of.lpstrFile);
	             else ret = Player->LoadSong(Name);

	if(!ret) {
		MessageBox(hDialog, "Error loading AHX tune!", "Error", MB_OK|MB_ICONERROR);
		return;
	}

	Player->InitSubsong(0);
	Subsong = 0;

	SyncButtons(1);
	InfoDialog.Load(Player);
	
	VERIFY(WaveOut.Play(Player));

	RenderTime(); DrawTime(); SyncTitle();
	SetTimer(hDialog, 0xbaba, 1000, NULL);
}

void AHXDialog::CloseSong()
{
	WaveOut.Stop();
	SyncButtons(0);
	InfoDialog.Unload();
	delete Player; Player = 0;
	KillTimer(hDialog, 0xbaba); iTime = -1;
	RenderTime(); DrawTime(); SyncTitle();
}

void AHXDialog::SyncButtons(int Mode)
{
	char title[256];

	if(Mode) {
		if(Player->Song.SubsongNr > 0) sprintf(title, "%s (%d/%d)", Player->Song.Name, Subsong, Player->Song.SubsongNr);
		                          else  strcpy(title, Player->Song.Name);
	} else strcpy(title, "<no song loaded>");
	
	EnableWindow(GetDlgItem(hDialog, IDC_PLAYPAUSE), Mode);
	EnableWindow(GetDlgItem(hDialog, IDC_STOP),      Mode);
	EnableWindow(GetDlgItem(hDialog, IDC_CLOSE),     Mode);
	EnableWindow(GetDlgItem(hDialog, IDC_BACKWD),    Mode);
	EnableWindow(GetDlgItem(hDialog, IDC_FWD),       Mode);
	SetWindowText(GetDlgItem(hDialog, IDC_NAME),     title);
	EnableWindow(GetDlgItem(hDialog, IDC_NAME)      ,Mode);
	EnableWindow(GetDlgItem(hDialog, IDC_VOLUME)    ,Mode);
	EnableWindow(GetDlgItem(hDialog, IDC_SKIPFWD)   ,Mode && Subsong < Player->Song.SubsongNr);
	EnableWindow(GetDlgItem(hDialog, IDC_SKIPBACK)  ,Mode && Subsong > 0);
}

void AHXDialog::RenderTime()
{
	if(WaveOut.Player)
		sprintf(sTime, "%02d:%02d", (WaveOut.Player->PlayingTime/50)/60, (WaveOut.Player->PlayingTime/50)%60);
	else
		sprintf(sTime, "--:--");

	HDC hDC = CreateCompatibleDC(NULL);
	HDC FontDC = CreateCompatibleDC(NULL);
	HBITMAP Old1 = (HBITMAP)SelectObject(hDC, hTimeBitmap);
	HBITMAP Old2 = (HBITMAP)SelectObject(FontDC, hTimeFont);

	HBRUSH Black = CreateSolidBrush(RGB(0,0,0));
	RECT r = { 0, 0, 77, 42 };
	FillRect(hDC, &r, Black);
	DeleteObject(Black);

	BitBlt(hDC, 7+14*2-3,9-2,5,24, FontDC,155,0,SRCCOPY); //doppelpunkt
	if(iTime == -1) {
		BitBlt(hDC, 7-2,9-2,13,24,FontDC,160,0,SRCCOPY);
		BitBlt(hDC, 7+14-2,9-2,13,24,FontDC,160,0,SRCCOPY);
		BitBlt(hDC, 7+14*2+6-2,9-2,13,24,FontDC,160,0,SRCCOPY);
		BitBlt(hDC, 7+14*3+6-2,9-2,13,24,FontDC,160,0,SRCCOPY);
	} else {
		BitBlt(hDC, 7-2,9-2,13,24,FontDC,14*(sTime[0]-'0'),0,SRCCOPY);
		BitBlt(hDC, 7+14-2,9-2,13,24,FontDC,14*(sTime[1]-'0'),0,SRCCOPY);
		BitBlt(hDC, 7+14*2+6-2,9-2,13,24,FontDC,14*(sTime[3]-'0'),0,SRCCOPY);
		BitBlt(hDC, 7+14*3+6-2,9-2,13,24,FontDC,14*(sTime[4]-'0'),0,SRCCOPY);
	}
	SelectObject(hDC, Old1);
	SelectObject(FontDC, Old2);
	DeleteDC(hDC); DeleteDC(FontDC);
}

void AHXDialog::DrawTime()
{
	HDC hDC = GetDC(GetDlgItem(hDialog, IDC_TIME));
	RECT r; GetClientRect(GetDlgItem(hDialog, IDC_TIME), &r);
	DrawState(hDC, NULL, NULL, (LPARAM)hTimeBitmap, 0, 2, 2, 77-4, 42-4, DST_BITMAP|DSS_NORMAL);
	ReleaseDC(GetDlgItem(hDialog, IDC_TIME), hDC);
}

void AHXDialog::DrawNoFocusButton(UINT id, int bm, LPDRAWITEMSTRUCT lpdi)
{
	UINT x, y, dx, dy;
	UINT state = lpdi->itemState;
	HDC hDC = lpdi->hDC;
	RECT r;
	GetClientRect(GetDlgItem(hDialog, id), &r);

	x = r.left; y = r.top; dx = r.right - r.left; dy = r.bottom - r.left;

	if(bm < 1000) {
		HBRUSH h3dface = CreateSolidBrush(GetSysColor(COLOR_3DFACE));
		FillRect(hDC, &r, h3dface);
		DeleteObject(h3dface);
	
		POINT p;
		SIZE s;
		if(bm >= 0) {
			BITMAP bmp;
			GetObject(hBtnBitmaps[bm], sizeof(BITMAP), &bmp);
			p.x=x+1+(dx-2-bmp.bmWidth)/2;
			p.y=y+1+(dy-2-bmp.bmHeight)/2;
			if(state&ODS_SELECTED) { p.x++; p.y++; }
			s.cx=dx-2-(dx-2-bmp.bmWidth)/2;
			s.cy=dy-2-(dy-2-bmp.bmHeight)/2;
			DrawState(hDC, NULL, NULL, (LPARAM)hBtnBitmaps[bm], 0, p.x, p.y, s.cx, s.cy, (state&ODS_DISABLED)?(DST_BITMAP|DSS_DISABLED):(DST_BITMAP|DSS_NORMAL));
		} else {
			p.x=x+(dx-16)/2;
			p.y=y+(dy-16)/2;
			if(state&ODS_SELECTED) { p.x++; p.y++; }
			s.cx=dx-(dx-16)/2;
			s.cy=dy-(dy-16)/2;
			DrawState(hDC, NULL, NULL, (LPARAM)hBtnIcons[-bm-1], 0, p.x, p.y, s.cx, s.cy, (state&ODS_DISABLED)?(DST_ICON|DSS_DISABLED):(DST_ICON|DSS_NORMAL));
		}
		DrawEdge(hDC, &r, (state&ODS_SELECTED)?EDGE_SUNKEN:EDGE_RAISED, BF_RECT);
	} else {
		DrawState(hDC, NULL, NULL, (LPARAM)hTimeBitmap, 0, 2, 2, 77-4, 42-4, DST_BITMAP|DSS_NORMAL);
		DrawEdge(hDC, &r, (state&ODS_SELECTED)?EDGE_SUNKEN:EDGE_RAISED, BF_RECT);
	}
}

#define SCOPE_ZOOM 4

void AHXDialog::RenderScope()
{
/*	HDC memDC = CreateCompatibleDC(NULL);
	HBITMAP Old1 = SelectObject(memDC, hTimeBitmap);

	HBRUSH Black = CreateSolidBrush(RGB(0,0,0));
	HBRUSH OldBrush = SelectObject(memDC, Black);
	HPEN Pen = CreatePen(PS_SOLID, 0, RGB(255, 255, 255));
	HPEN OldPen = SelectObject(memDC, Pen);
	
	RECT r = { 0, 0, 77, 42 };
	FillRect(memDC, &r, Black);


	if(WaveOut.Playing)
		if(WaveOut.Bits == 16) {
			short SampleBuffer[77*4], *b = SampleBuffer;
			int SampleValid = 77*4*2;

			if(WaveOut.CopyBuffer((void*)SampleBuffer, &SampleValid)) {
				MoveToEx(memDC, -1, -1 ,NULL);
				for(int i = 0; i < 77; i++) {
					int y = *b/(32768/16)+19;
					b += SCOPE_ZOOM;
					LineTo(memDC, i-1, y);
				}
			}
		} else if(WaveOut.Bits == 8) {
			unsigned char* SampleBuffer;
			unsigned char* SampleBuffer2;
			int SampleValid, SampleValid2;

//			if(WaveOut.LockBuffer((void**)&SampleBuffer, (void**)&SampleBuffer2, &SampleValid, &SampleValid2)) {
				unsigned char* b = SampleBuffer;
				MoveToEx(memDC, -1, -1 ,NULL);
				for(int i = 0; i/2 < 77 && i < 0x280 && SampleValid > 0; i += 2, SampleValid -= 2*SCOPE_ZOOM) {
					int x = i/2;
					int y = *b/8+6;
					b += SCOPE_ZOOM;
					LineTo(memDC, x-1, y);
				}
				if(i < 0x280 && SampleBuffer2) {
					b = SampleBuffer2;
					for(; i/2 < 77 && i < 0x280; i += 2) {
						int x = i/2;
						int y = *b / 8+6;
						b += SCOPE_ZOOM;
						LineTo(memDC, x-1, y);
					}
				}
//				WaveOut.UnlockBuffer();
//			}
		}
	SelectObject(memDC, Old1);
	SelectObject(memDC, OldBrush);
	SelectObject(memDC, OldPen);
	DeleteDC(memDC); DeleteObject(Black); DeleteObject(Pen);
*/}

void FAR PASCAL AHXDialog::_TimerCallback(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	AHXDialog* This = (AHXDialog*)dwUser;
	This->RenderScope();
	This->DrawTime();
}

long FAR PASCAL AHXDialog::_AHXDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static AHXDialog* This = NULL;
	if(message == WM_INITDIALOG) This = (AHXDialog*)lParam;
	return This->AHXDlgProc(hDlg, message, wParam, lParam);
}
