#ifndef __MAIN_H
#define __MAIN_H

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>

#include "ahx/AHX.h"
#include "ahx/Windows.h"

class AHXInfoDialog {
public:
	void Init(HWND hParent);
	void Show();

	void Unload();
	void Load(AHXPlayer* Player);

	HWND hInfoDlg;
	int bVisible;

protected:
	long InfoDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
	static long FAR PASCAL _InfoDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
};

class AHXProperties {
public:
	 AHXProperties();
	~AHXProperties();

	void Show(HWND hParent);
	void Sync(int i, HWND hDlg);
	void CheckRegKeys(int* i1, int* i2);
	void SetRegKeys();
	void ClearRegKeys();

	int MixRate, t_MixRate; // 0 - 3 = 8, 11, 22, 44 KHz
	int BufSize, t_BufSize; // 6 - 32 = # of 2frames (40ms) blocks
	int Oversampling, t_Oversampling; // 0 = off, 1 = simple, ...
	int Output, t_Output;  // 0 = 8bit mono, 1 = 16bit mono
	int Boost, t_Boost; // (int)(fBoost * 10), e.g. 10 = 1.0; 40 = 4.0
	int Dock, t_Dock;

protected:
	BOOL Prop1DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	BOOL Prop2DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	BOOL Prop3DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static BOOL CALLBACK _Prop1DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static BOOL CALLBACK _Prop2DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static BOOL CALLBACK _Prop3DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
};

class AHXAboutDialog {
public:
	void Show(HWND hParent);

protected:
	void InitScroller();
	void RenderScroller();
	
	BOOL DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static BOOL CALLBACK _DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static void FAR PASCAL _TimerCallback(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2);

	HWND hDialog;
	
	int iTimerId, iTimerRes;

	HBITMAP hScroller;
	int iScrollPos;
};

class AHXDialog {
public:
	void Init();

protected:
	void DrawNoFocusButton(UINT id, int bm, LPDRAWITEMSTRUCT lpdi);
	void RenderTime();
	void RenderScope();
	void DrawTime();
	void LoadSong(char* Name = "");
	void CloseSong();
	void SyncButtons(int Mode);
	void SyncTitle();
	
	int Subsong;
	
	HBITMAP hBtnBitmaps[8]; HBITMAP hTimeBitmap; HBITMAP hTimeFont;
	HICON   hBtnIcons[2];
	HWND    hDialog;

	int  iTimeMode;
	
	int  iTimerRes, iTimerId;

	int  iTime;
	char sTime[8];

	AHXWaves Waves;
	AHXPlayer* Player;
	AHXWaveOut WaveOut;

	long AHXDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
	static long FAR PASCAL _AHXDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
	static void FAR PASCAL _TimerCallback(UINT wTimerID, UINT msg, DWORD dwUser, DWORD dw1, DWORD dw2);

	friend class AHXProperties;
};

#endif