#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <commctrl.h>

#include "ahx/AHX.h"
#include "ahx/Windows.h"
#include "Debug.h"
#include "Main.h"
#include "resource.h"

extern AHXDialog Dialog;

AHXProperties::AHXProperties()
{
	char Ini[256];

	GetModuleFileName(NULL, Ini, 256); strcpy(&Ini[strlen(Ini)-3], "ini");

	MixRate = GetPrivateProfileInt("Output", "MixRate", 3, Ini);
	BufSize = GetPrivateProfileInt("Output", "BufSize", 16, Ini);
	Oversampling = GetPrivateProfileInt("Output", "Oversampling", 0, Ini);
	Output = GetPrivateProfileInt("Output", "Output", 1, Ini);
	Boost = GetPrivateProfileInt("Output", "Boost", 15, Ini);

	Dock = GetPrivateProfileInt("Info Window", "Dock", 1, Ini);
}

AHXProperties::~AHXProperties()
{
	char Ini[256], Str[64];

	GetModuleFileName(NULL, Ini, 256); strcpy(&Ini[strlen(Ini)-3], "ini");

	_snprintf(Str, 64, "%d", MixRate); WritePrivateProfileString("Output", "MixRate", Str, Ini);
	_snprintf(Str, 64, "%d", BufSize); WritePrivateProfileString("Output", "BufSize", Str, Ini);
	_snprintf(Str, 64, "%d", Oversampling); WritePrivateProfileString("Output", "Oversampling", Str, Ini);
	_snprintf(Str, 64, "%d", Output); WritePrivateProfileString("Output", "Output", Str, Ini);
	_snprintf(Str, 64, "%d", Boost); WritePrivateProfileString("Output", "Boost", Str, Ini);

	_snprintf(Str, 64, "%d", Dock); WritePrivateProfileString("Info Window", "Dock", Str, Ini);
}

void AHXProperties::Show(HWND hParent)
{
    PROPSHEETPAGE psp[3];
    PROPSHEETHEADER psh;

    psp[0].dwSize = sizeof(PROPSHEETPAGE);
    psp[0].dwFlags = PSP_USETITLE;
    psp[0].hInstance = GetModuleHandle(NULL);
    psp[0].pszTemplate = MAKEINTRESOURCE(IDD_PROPPAGE1);
    psp[0].pszIcon = 0;
    psp[0].pfnDlgProc = (DLGPROC)_Prop1DialogProc;
    psp[0].pszTitle = "Output";
    psp[0].lParam = (LPARAM)this;
    psp[0].pfnCallback = NULL;

    psp[1].dwSize = sizeof(PROPSHEETPAGE);
    psp[1].dwFlags = PSP_USETITLE;
    psp[1].hInstance = GetModuleHandle(NULL);
    psp[1].pszTemplate = MAKEINTRESOURCE(IDD_PROPPAGE2);
    psp[1].pszIcon = 0;
    psp[1].pfnDlgProc = (DLGPROC)_Prop2DialogProc;
    psp[1].pszTitle = "Shell";
    psp[1].lParam = (LPARAM)this;
    psp[1].pfnCallback = NULL;

    psp[2].dwSize = sizeof(PROPSHEETPAGE);
    psp[2].dwFlags = PSP_USETITLE;
    psp[2].hInstance = GetModuleHandle(NULL);
    psp[2].pszTemplate = MAKEINTRESOURCE(IDD_PROPPAGE3);
    psp[2].pszIcon = 0;
    psp[2].pfnDlgProc = (DLGPROC)_Prop3DialogProc;
    psp[2].pszTitle = "User Interface";
    psp[2].lParam = (LPARAM)this;
    psp[2].pfnCallback = NULL;

    psh.dwSize = sizeof(PROPSHEETHEADER);
    psh.dwFlags = PSH_PROPSHEETPAGE;
    psh.hwndParent = hParent;
    psh.hInstance = GetModuleHandle(NULL);
    psh.pszCaption = (LPSTR) "Options...";
    psh.nPages = sizeof(psp) / sizeof(psp[0]);
    psh.nStartPage = 0;
    psh.ppsp = (LPCPROPSHEETPAGE) &psp;
    psh.pfnCallback = NULL;

    PropertySheet(&psh);
}

BOOL CALLBACK AHXProperties::_Prop1DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
	static AHXProperties* This = NULL;
	if(uMsg == WM_INITDIALOG) This = (AHXProperties*)(((PROPSHEETPAGE*)lParam)->lParam);
	return This->Prop1DialogProc(hDlg, uMsg, wParam, lParam);
}

BOOL CALLBACK AHXProperties::_Prop2DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
	static AHXProperties* This = NULL;
	if(uMsg == WM_INITDIALOG) This = (AHXProperties*)(((PROPSHEETPAGE*)lParam)->lParam);
	return This->Prop2DialogProc(hDlg, uMsg, wParam, lParam);
}

BOOL CALLBACK AHXProperties::_Prop3DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
	static AHXProperties* This = NULL;
	if(uMsg == WM_INITDIALOG) This = (AHXProperties*)(((PROPSHEETPAGE*)lParam)->lParam);
	return This->Prop3DialogProc(hDlg, uMsg, wParam, lParam);
}

BOOL AHXProperties::Prop1DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static char* hz[4] = { "8 kHz", "11 kHz", "22 kHz", "44 kHz" };
	char xx[16];

	switch (uMsg) {
		case WM_INITDIALOG:
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER1), TBM_SETRANGE, TRUE, MAKELONG(6, 32));
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER1), TBM_SETLINESIZE, 0, 1);
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER1), TBM_SETPAGESIZE, 0, 2);
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER1), TBM_SETPOS, TRUE, BufSize);
			_snprintf(xx, 16, "%d ms", BufSize*40); SetWindowText(GetDlgItem(hDlg, IDC_BUFFERSIZE), xx);

			SendMessage(GetDlgItem(hDlg, IDC_SLIDER2), TBM_SETRANGE, TRUE, MAKELONG(0, 3));
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER2), TBM_SETLINESIZE, 0, 1);
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER2), TBM_SETPAGESIZE, 0, 1);
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER2), TBM_SETPOS, TRUE, MixRate);
			SetWindowText(GetDlgItem(hDlg, IDC_QUALITY), hz[MixRate]);

			SendMessage(GetDlgItem(hDlg, IDC_SLIDER3), TBM_SETRANGE, TRUE, MAKELONG(10, 40));
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER3), TBM_SETLINESIZE, 0, 1);
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER3), TBM_SETPAGESIZE, 0, 5);
			SendMessage(GetDlgItem(hDlg, IDC_SLIDER3), TBM_SETPOS, TRUE, Boost);
			_snprintf(xx, 16, "%.1f", (float)Boost/10.f); SetWindowText(GetDlgItem(hDlg, IDC_BOOST), xx);

			Button_SetCheck(GetDlgItem(hDlg, IDC_8BIT), Output == 0);
			Button_SetCheck(GetDlgItem(hDlg, IDC_16BIT), Output == 1);

			Button_SetCheck(GetDlgItem(hDlg, IDC_OS1), Oversampling == 1);
			Button_SetCheck(GetDlgItem(hDlg, IDC_OS0), Oversampling == 0);

			t_BufSize = BufSize;
			t_MixRate = MixRate;
			t_Boost = Boost;
			t_Oversampling = Oversampling;
			t_Output = Output;
			break;
		case WM_HSCROLL: {
			HWND hScroller = (HWND)lParam;
			int dwPos = SendMessage(hScroller, TBM_GETPOS, 0, 0);

			if(hScroller == GetDlgItem(hDlg, IDC_SLIDER1)) {
				t_BufSize = dwPos;
				_snprintf(xx, 16, "%d ms", t_BufSize*40);
				SetWindowText(GetDlgItem(hDlg, IDC_BUFFERSIZE), xx);
			} else if(hScroller == GetDlgItem(hDlg, IDC_SLIDER2)) {
				t_MixRate = dwPos;
				SetWindowText(GetDlgItem(hDlg, IDC_QUALITY), hz[t_MixRate]);
			} else if(hScroller == GetDlgItem(hDlg, IDC_SLIDER3)) {
				t_Boost = dwPos;
				_snprintf(xx, 16, "%.1f", (float)t_Boost/10.f); SetWindowText(GetDlgItem(hDlg, IDC_BOOST), xx);
			}
			Sync(0, hDlg);
		} break;
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case IDC_8BIT:
				case IDC_16BIT:
					t_Output = Button_GetCheck(GetDlgItem(hDlg, IDC_16BIT));
					break;
				case IDC_OS0:
				case IDC_OS1:
					t_Oversampling = Button_GetCheck(GetDlgItem(hDlg, IDC_OS1));
					break;
			}
			Sync(0, hDlg);
			break;
		case WM_NOTIFY: {
			switch (((NMHDR FAR *)lParam)->code) {
				case PSN_APPLY: {
					if(BufSize != t_BufSize || MixRate != t_MixRate || Output != t_Output) {
						int Playing = Dialog.WaveOut.Playing;
						int Paused = Dialog.WaveOut.Paused;
						Dialog.WaveOut.StopBackgroundPlay();
						Dialog.WaveOut.Free();
						int MixRates[4] = { 8000, 11025, 22050, 44100 };
						CHECK(Dialog.WaveOut.Init(MixRates[t_MixRate], (t_Output+1)*8, 2, t_BufSize, ((float)t_Boost)*.1f), "Can't initialize Wave Output");
						CHECK(Dialog.WaveOut.StartBackgroundPlay(), "Couldn't start background play");
						if(Playing) Dialog.WaveOut.Play(Dialog.Player);
						if(Playing && Paused) Dialog.WaveOut.Pause();
					} else {
						Dialog.WaveOut.SetOption(AHXOF_BOOST, ((float)t_Boost)*.1f);
					}
					if(Oversampling != t_Oversampling) {
						Dialog.WaveOut.SetOption(AHXOI_OVERSAMPLING, t_Oversampling);
					}

					BufSize = t_BufSize;
					MixRate = t_MixRate;
					Boost = t_Boost;
					Oversampling = t_Oversampling;
					Output = t_Output;

					Sync(0, hDlg);
					return SetWindowLong(hDlg, DWL_MSGRESULT, FALSE);
				}
				case PSN_RESET: {		// Cancel Button pressed
					return SetWindowLong(hDlg, DWL_MSGRESULT, FALSE);
				}
			}
			break; //END OF WM_NOTIFY
		}
	}
	return SetWindowLong(hDlg, DWL_MSGRESULT, TRUE);
}

void AHXProperties::Sync(int i, HWND hDlg)
{
	int changed;
	switch(i) {
		case 0: {
			changed = (MixRate != t_MixRate) || (BufSize != t_BufSize) || 
			          (Oversampling != t_Oversampling) || (Output != t_Output) || (Boost != t_Boost);
		} break;
		case 2: {
			changed = (Dock != t_Dock);
		} break;
	}
	if(changed) PropSheet_Changed(GetParent(hDlg), hDlg);
	       else PropSheet_UnChanged(GetParent(hDlg), hDlg);
}

void AHXProperties::CheckRegKeys(int* i1, int* i2)
{
	HKEY key;

	if(RegOpenKeyEx(HKEY_CLASSES_ROOT,".AHX",0,KEY_ALL_ACCESS,&key)==ERROR_SUCCESS) {
		*i1 = TRUE;
		RegCloseKey(key);
	} else *i1 = FALSE;
	if(RegOpenKeyEx(HKEY_CLASSES_ROOT,"AHX.Song",0,KEY_ALL_ACCESS,&key)==ERROR_SUCCESS) {
		*i2 = TRUE;
		RegCloseKey(key);
	} else *i2 = FALSE;
}

void AHXProperties::SetRegKeys()
{
	HKEY key,key0,key1,key2,key3;
	DWORD result;
	char cc[300];
	char bb[300];
	RegCreateKeyEx(HKEY_CLASSES_ROOT,"AHX.Song",0,"String",REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&key,&result);
	RegSetValueEx(key,NULL,0,REG_SZ,(const unsigned char*)"AHX Song",33);
	GetModuleFileName(NULL,cc,280);
	strcat(cc,",0");
	RegCreateKeyEx(key,"DefaultIcon",0,"String",REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&key0,&result);
	RegSetValueEx(key0,NULL,0,REG_SZ,(const unsigned char*)cc,strlen(cc)+1);
	RegCloseKey(key0);
	RegCreateKeyEx(key,"shell",0,"String",REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&key1,&result);
	RegCreateKeyEx(key1,"open",0,"String",REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&key2,&result);
	RegCreateKeyEx(key2,"command",0,"String",REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&key3,&result);
	bb[0]='"';
	GetModuleFileName(NULL,bb+1,280);
	strcat(bb,"\" \"%1\"");
	RegSetValueEx(key3,NULL,0,REG_SZ,(const unsigned char*)bb,strlen(bb)+1);
	RegCloseKey(key3);
	RegCloseKey(key2);
	RegCloseKey(key1);
	RegCloseKey(key);

	RegCreateKeyEx(HKEY_CLASSES_ROOT,".AHX",0,"String",REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&key,&result);
	RegSetValueEx(key,NULL,0,REG_SZ,(const unsigned char *)"AHX.Song",12);
	RegCloseKey(key);
}

void AHXProperties::ClearRegKeys()
{
	HKEY key;

	if(RegOpenKeyEx(HKEY_CLASSES_ROOT,".AHX",0,KEY_ALL_ACCESS,&key)==ERROR_SUCCESS) {
		RegDeleteValue(key,NULL);
		RegCloseKey(key);
		RegDeleteKey(HKEY_CLASSES_ROOT,".AHX");
	}
	if(RegOpenKeyEx(HKEY_CLASSES_ROOT,"AHX.Song",0,KEY_ALL_ACCESS,&key)==ERROR_SUCCESS) {
		RegDeleteValue(key,NULL);
		RegCloseKey(key);
		RegDeleteKey(HKEY_CLASSES_ROOT,"AHX.Song");
	}
}

BOOL AHXProperties::Prop2DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
		case WM_INITDIALOG: {
			int i1, i2;
			CheckRegKeys(&i1, &i2);
			Button_SetCheck(GetDlgItem(hDlg, IDC_CHECK1), i1);
			Button_SetCheck(GetDlgItem(hDlg, IDC_CHECK2), i2);
		} break;
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case IDC_REGISTER: {
					SetRegKeys();
					int i1, i2;
					CheckRegKeys(&i1, &i2);
					Button_SetCheck(GetDlgItem(hDlg, IDC_CHECK1), i1);
					Button_SetCheck(GetDlgItem(hDlg, IDC_CHECK2), i2);
					break;
				}
				case IDC_UNREGISTER: {
					ClearRegKeys();
					int i1, i2;
					CheckRegKeys(&i1, &i2);
					Button_SetCheck(GetDlgItem(hDlg, IDC_CHECK1), i1);
					Button_SetCheck(GetDlgItem(hDlg, IDC_CHECK2), i2);
				} break;
			}
			break;
		}
	return SetWindowLong(hDlg, DWL_MSGRESULT, TRUE);
}

BOOL AHXProperties::Prop3DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg) {
		case WM_INITDIALOG: {
			t_Dock = Dock;
			Button_SetCheck(GetDlgItem(hDlg, IDC_DOCK), t_Dock);
		} break;
		case WM_COMMAND:
			switch(LOWORD(wParam)) {
				case IDC_DOCK: {
					t_Dock = Button_GetCheck(GetDlgItem(hDlg, IDC_DOCK));
				} break;
			}
			Sync(2, hDlg);
			break;
		case WM_NOTIFY: {
			switch (((NMHDR FAR *)lParam)->code) {
				case PSN_APPLY: {
					Dock = t_Dock;

					Sync(2, hDlg);
					return SetWindowLong(hDlg, DWL_MSGRESULT, FALSE);
				}
				case PSN_RESET: {		// Cancel Button pressed
					return SetWindowLong(hDlg, DWL_MSGRESULT, FALSE);
				}
			}
			break; //END OF WM_NOTIFY
		}
	}
	return SetWindowLong(hDlg, DWL_MSGRESULT, TRUE);
}
